@@ -120,7 +120,7 @@ static void process_entries(const void *nodep, VISIT value, int level unused)
 	len = strlen((*t)->from);
 
 	list_for_each(target_list, entry) {
-		if ((entry->mode & AA_SHARED_PERMS) || entry->alias_ignore)
+		if ((entry->perms & AA_SHARED_PERMS) || entry->alias_ignore)
 			continue;
 		if (entry->name && strncmp((*t)->from, entry->name, len) == 0) {
 			char *n = do_alias(*t, entry->name);
@@ -141,7 +141,7 @@ static void process_entries(const void *nodep, VISIT value, int level unused)
 			dup->link_name = n;
 		}
 		if (dup) {
-			dup->alias_ignore = 1;
+			dup->alias_ignore = true;
 			/* adds to the front of the list, list iteratition
 			 * will skip it
 			 */
