@@ -31,28 +31,35 @@
 
 typedef set<int> Signals;
 
-int parse_signal_mode(const char *str_mode, int *mode, int fail);
+int find_signal_mapping(const char *sig);
+int parse_signal_perms(const char *str_perms, perm32_t *perms, int fail);
 
-class signal_rule: public rule_t {
+class signal_rule: public perms_rule_t {
 	void extract_sigs(struct value_list **list);
 	void move_conditionals(struct cond_entry *conds);
 public:
 	Signals signals;
 	char *peer_label;
-	int mode;
-	int audit;
-	int deny;
 
-	signal_rule(int mode, struct cond_entry *conds);
+	signal_rule(perm32_t perms, struct cond_entry *conds);
 	virtual ~signal_rule() {
 		signals.clear();
 		free(peer_label);
 	};
+	virtual bool valid_prefix(const prefixes &p, const char *&error) {
+		if (p.owner != OWNER_UNSPECIFIED) {
+			error = "owner prefix not allowed on signal rules";
+			return false;
+		}
+		return true;
+	};
 
 	virtual ostream &dump(ostream &os);
 	virtual int expand_variables(void);
 	virtual int gen_policy_re(Profile &prof);
-	virtual void post_process(Profile &prof unused) { };
+
+	virtual bool is_mergeable(void) { return true; }
+	virtual int cmp(rule_t const &rhs) const;
 
 protected:
 	virtual void warn_once(const char *name) override;
