@@ -18,7 +18,21 @@ import sys
 import time
 
 import LibAppArmor
+import apparmor.ui as aaui
 from apparmor.common import AppArmorBug, AppArmorException, DebugLogger, hasher, open_file_read, split_name
+from apparmor.rule.capability import CapabilityRule
+from apparmor.rule.change_profile import ChangeProfileRule
+from apparmor.rule.dbus import DbusRule
+from apparmor.rule.file import FileRule
+from apparmor.rule.io_uring import IOUringRule
+from apparmor.rule.mount import MountRule
+from apparmor.rule.mqueue import MessageQueueRule
+from apparmor.rule.network import NetworkRule
+from apparmor.rule.pivot_root import PivotRootRule
+from apparmor.rule.ptrace import PtraceRule
+from apparmor.rule.signal import SignalRule
+from apparmor.rule.unix import UnixRule
+from apparmor.rule.userns import UserNamespaceRule
 from apparmor.translations import init_translation
 
 _ = init_translation()
@@ -29,6 +43,22 @@ class ReadLog:
     # used to pre-filter log lines so that we hand over only relevant lines to LibAppArmor parsing
     RE_LOG_ALL = re.compile('apparmor=|operation=|type=AVC')
 
+    ruletypes = {
+        'capability': CapabilityRule,
+        'change_profile': ChangeProfileRule,
+        'dbus': DbusRule,
+        'file': FileRule,
+        'ptrace': PtraceRule,
+        'signal': SignalRule,
+        'userns': UserNamespaceRule,
+        'mqueue': MessageQueueRule,
+        'io_uring': IOUringRule,
+        'mount': MountRule,
+        'unix': UnixRule,
+        'network': NetworkRule,
+        'pivot_root': PivotRootRule,
+    }
+
     def __init__(self, filename, active_profiles, profile_dir):
         self.filename = filename
         self.profile_dir = profile_dir
@@ -53,14 +83,16 @@ class ReadLog:
             'change_profile': {},  # flat, no hasher needed  (at least in logparser which doesn't support EXEC MODE and EXEC COND)
             'dbus':         hasher(),
             'exec':         hasher(),
+            'file':         hasher(),
             'network':      hasher(),
-            'path':         hasher(),
+            'pivot_root':   hasher(),
             'ptrace':       hasher(),
             'signal':       hasher(),
             'userns':       hasher(),
             'mqueue':       hasher(),
             'io_uring':     hasher(),
             'mount':        hasher(),
+            'unix':         hasher(),
         }
 
     def prefetch_next_log_entry(self):
@@ -80,11 +112,66 @@ class ReadLog:
         self.next_log_entry = None
         return log_entry
 
-    def parse_event(self, msg):
-        """Parse the event from log into key value pairs"""
-        msg = msg.strip()
-        self.debug_logger.info('parse_event: %s', msg)
-        event = LibAppArmor.parse_record(msg)
+    def get_event_type(self, e):
+
+        if e['operation'] == 'exec':
+            return 'file'
+        elif e['class'] and e['class'] == 'namespace':
+            if e['denied_mask'] and e['denied_mask'].startswith('userns_'):
+                return 'userns'
+            elif not e['denied_mask'] and e['request_mask'].startswith('userns_'):  # To support transition to special userns profiles
+                return 'userns'
+        elif e['class'] and e['class'].endswith('mqueue'):
+            return 'mqueue'
+        elif e['class'] and e['class'] == 'io_uring':
+            return 'io_uring'
+        elif e['class'] and e['class'] == 'mount' or e['operation'] == 'mount':
+            return 'mount'
+        elif e['operation'] and e['operation'] == 'pivotroot':
+            return 'pivot_root'
+        elif e['class'] and e['class'] == 'net' and e['family'] and e['family'] == 'unix':
+            return 'unix'
+        elif e['class'] == 'file' or self.op_type(e) == 'file':
+            return 'file'
+        elif e['operation'] == 'capable':
+            return 'capability'
+        elif self.op_type(e) == 'net':
+            return 'network'
+        elif e['operation'] == 'change_hat':
+            return 'change_hat'
+        elif e['operation'] == 'change_profile':
+            return 'change_profile'
+        elif e['operation'] == 'ptrace':
+            return 'ptrace'
+        elif e['operation'] == 'signal':
+            return 'signal'
+        elif e['operation'] and e['operation'].startswith('dbus_'):
+            return 'dbus'
+
+        else:
+            self.debug_logger.debug('UNHANDLED: %s', e)
+
+        return None
+
+    def get_rule_type(self, r):
+        for k, v in self.ruletypes.items():
+            if v.match(r):
+                return k, v
+        return None
+
+    def create_rule_from_ev(self, ev):
+        event_type = self.get_event_type(ev)
+        if not event_type:
+            return None
+        ruletype = self.ruletypes[event_type]
+
+        try:
+            return ruletype.create_from_ev(ev)
+        except (AppArmorException, AppArmorBug):
+            return None
+
+    def parse_record(self, event):
+        """Parse the record from LibAppArmor into key value pairs"""
         ev = dict()
         ev['resource'] = event.info
         ev['active_hat'] = event.active_hat
@@ -112,29 +199,49 @@ class ReadLog:
             ev['fsuid'] = event.fsuid
             ev['ouid'] = event.ouid
 
-        if ev['operation'] and ev['operation'] == 'signal':
+        event_type = self.get_event_type(ev)
+        if event_type == 'signal':
             ev['signal'] = event.signal
             ev['peer'] = event.peer
-        elif ev['operation'] and ev['operation'] == 'ptrace':
+        elif event_type == 'ptrace':
             ev['peer'] = event.peer
-        elif ev['operation'] and ev['operation'] == 'mount':
-            ev['flags'] = event.flags
-            ev['fs_type'] = event.fs_type
+        elif event_type == 'pivot_root':
             ev['src_name'] = event.src_name
-        elif ev['operation'] and (ev['operation'] == 'umount'):
+        elif event_type == 'mount':
             ev['flags'] = event.flags
             ev['fs_type'] = event.fs_type
-        elif ev['operation'] and ev['operation'].startswith('dbus_'):
+            if ev['operation'] and ev['operation'] == 'mount':
+                ev['src_name'] = event.src_name  # mount can have a source but not umount.
+        elif event_type == 'userns':
+            ev['execpath'] = event.execpath
+            ev['comm'] = event.comm
+        elif event_type == 'network':
+            ev['accesses'] = event.requested_mask
+            ev['port'] = event.net_local_port or None
+            ev['remote_port'] = event.net_foreign_port or None
+            ev['addr'] = event.net_local_addr
+            ev['peer_addr'] = event.net_foreign_addr
+            ev['addr'] = event.net_local_addr
+            ev['peer_addr'] = event.net_foreign_addr
+        elif event_type == 'unix':
+            ev['accesses'] = event.requested_mask
+            ev['port'] = event.net_local_port or None
+            ev['remote_port'] = event.net_foreign_port or None
+            ev['addr'] = event.net_addr
+            ev['peer_addr'] = event.peer_addr
+            ev['peer'] = event.peer
+            ev['peer_profile'] = event.peer_profile
+        elif event_type == 'dbus':
             ev['peer_profile'] = event.peer_profile
             ev['bus'] = event.dbus_bus
             ev['path'] = event.dbus_path
             ev['interface'] = event.dbus_interface
             ev['member'] = event.dbus_member
 
-        elif ev['operation'] and ev['operation'].startswith('uring_'):
+        elif event_type == 'io_uring':
             ev['peer_profile'] = event.peer_profile
-
-        LibAppArmor.free_record(event)
+        elif event_type == 'capability':
+            ev['comm'] = event.comm
 
         if not ev['time']:
             ev['time'] = int(time.time())
@@ -164,6 +271,20 @@ class ReadLog:
         else:
             return None
 
+    def parse_event(self, msg):
+        """Parse the event from log into key value pairs"""
+        msg = msg.strip()
+        self.debug_logger.info('parse_event: %s', msg)
+        try:
+            event = LibAppArmor.parse_record(msg)
+        except TypeError:
+            aaui.UI_Important(_("WARNING: Cannot process log message, skipping entry. Make sure log is plaintext."))
+            return None
+
+        ev = self.parse_record(event)
+        LibAppArmor.free_record(event)
+        return ev
+
     def parse_event_for_tree(self, e):
         aamode = e.get('aamode', 'UNKNOWN')
 
@@ -188,83 +309,43 @@ class ReadLog:
 
         if profile != 'null-complain-profile' and not self.profile_exists(profile):
             return
+        # TODO: replace all the if conditions with a loop over 'ruletypes'
         if e['operation'] == 'exec':
-            if not e['name']:
-                raise AppArmorException('exec without executed binary')
-
-            if not e['name2']:
-                e['name2'] = ''  # exec events in enforce mode don't have target=...
-
-            self.hashlog[aamode][full_profile]['exec'][e['name']][e['name2']] = True
+            FileRule.hashlog_from_event(self.hashlog[aamode][full_profile]['exec'], e)
             return
 
         elif e['class'] and e['class'] == 'namespace':
             if e['denied_mask'].startswith('userns_'):
-                self.hashlog[aamode][full_profile]['userns'][ e['denied_mask'][7:] ] = True  # [7:] removes the 'userns_' prefix
+                UserNamespaceRule.hashlog_from_event(self.hashlog[aamode][full_profile]['userns'], e)
             return
 
         elif e['class'] and e['class'].endswith('mqueue'):
-            mqueue_type = e['class'].partition('_')[0]
-            self.hashlog[aamode][full_profile]['mqueue'][e['denied_mask']][mqueue_type][e['name']] = True
+            MessageQueueRule.hashlog_from_event(self.hashlog[aamode][full_profile]['mqueue'], e)
             return
 
         elif e['class'] and e['class'] == 'io_uring':
-            self.hashlog[aamode][full_profile]['io_uring'][e['denied_mask']][e['peer_profile']] = True
+            IOUringRule.hashlog_from_event(self.hashlog[aamode][full_profile]['io_uring'], e)
             return
 
         elif e['class'] and e['class'] == 'mount' or e['operation'] == 'mount':
-            if e['flags'] != None:
-                e['flags'] = ('=', e['flags'])
-            if e['fs_type'] != None:
-                e['fs_type'] = ('=', e['fs_type'])
+            MountRule.hashlog_from_event(self.hashlog[aamode][full_profile]['mount'], e)
 
+        elif e['operation'] and e['operation'] == 'pivotroot':
+            PivotRootRule.hashlog_from_event(self.hashlog[aamode][full_profile]['pivot_root'], e)
 
-            if e['operation'] == 'mount':
-                self.hashlog[aamode][full_profile]['mount'][e['operation']][e['flags']][e['fs_type']][e['name']][e['src_name']] = True
-            else:  # Umount
-                self.hashlog[aamode][full_profile]['mount'][e['operation']][e['flags']][e['fs_type']][e['name']][None] = True
+        elif e['class'] and e['class'] == 'net' and e['family'] and e['family'] == 'unix':
+            UnixRule.hashlog_from_event(self.hashlog[aamode][full_profile]['unix'], e)
             return
 
-        elif self.op_type(e) == 'file':
-            # Map c (create) and d (delete) to w (logging is more detailed than the profile language)
-            dmask = e['denied_mask']
-            dmask = dmask.replace('c', 'w')
-            dmask = dmask.replace('d', 'w')
-
-            owner = False
-
-            if '::' in dmask:
-                # old log styles used :: to indicate if permissions are meant for owner or other
-                (owner_d, other_d) = dmask.split('::')
-                if owner_d and other_d:
-                    raise AppArmorException('Found log event with both owner and other permissions. Please open a bugreport!')
-                if owner_d:
-                    dmask = owner_d
-                    owner = True
-                else:
-                    dmask = other_d
-
-            if e.get('ouid') is not None and e['fsuid'] == e['ouid']:
-                # in current log style, owner permissions are indicated by a match of fsuid and ouid
-                owner = True
-
-            if 'x' in dmask and dmask != 'x':
-                dmask = dmask.replace('x', '')  # if dmask contains x and another mode, drop x here - we should see a separate exec event
-
-            for perm in dmask:
-                if perm in 'mrwalk':  # intentionally not allowing 'x' here
-                    self.hashlog[aamode][full_profile]['path'][e['name']][owner][perm] = True
-                else:
-                    raise AppArmorException(_('Log contains unknown mode %s') % dmask)
-
-            return
+        elif e['class'] == 'file' or self.op_type(e) == 'file':
+            FileRule.hashlog_from_event(self.hashlog[aamode][full_profile]['file'], e)
 
         elif e['operation'] == 'capable':
-            self.hashlog[aamode][full_profile]['capability'][e['name']] = True
+            CapabilityRule.hashlog_from_event(self.hashlog[aamode][full_profile]['capability'], e)
             return
 
         elif self.op_type(e) == 'net':
-            self.hashlog[aamode][full_profile]['network'][e['family']][e['sock_type']][e['protocol']] = True
+            NetworkRule.hashlog_from_event(self.hashlog[aamode][full_profile]['network'], e)
             return
 
         elif e['operation'] == 'change_hat':
@@ -275,7 +356,7 @@ class ReadLog:
             return
 
         elif e['operation'] == 'change_profile':
-            self.hashlog[aamode][full_profile]['change_profile'][e['name2']] = True
+            ChangeProfileRule.hashlog_from_event(self.hashlog[aamode][full_profile]['change_profile'], e)
             return
 
         elif e['operation'] == 'ptrace':
@@ -286,15 +367,15 @@ class ReadLog:
                 self.debug_logger.debug('ignored garbage ptrace event with empty denied_mask')
                 return
 
-            self.hashlog[aamode][full_profile]['ptrace'][e['peer']][e['denied_mask']] = True
+            PtraceRule.hashlog_from_event(self.hashlog[aamode][full_profile]['ptrace'], e)
             return
 
         elif e['operation'] == 'signal':
-            self.hashlog[aamode][full_profile]['signal'][e['peer']][e['denied_mask']][e['signal']] = True
+            SignalRule.hashlog_from_event(self.hashlog[aamode][full_profile]['signal'], e)
             return
 
         elif e['operation'] and e['operation'].startswith('dbus_'):
-            self.hashlog[aamode][full_profile]['dbus'][e['denied_mask']][e['bus']][e['path']][e['name']][e['interface']][e['member']][e['peer_profile']] = True
+            DbusRule.hashlog_from_event(self.hashlog[aamode][full_profile]['dbus'], e)
             return
 
         else:
@@ -377,14 +458,14 @@ class ReadLog:
     def op_type(self, event):
         """Returns the operation type if known, unknown otherwise"""
 
-        if event['operation'] and (event['operation'].startswith('file_') or
-                                   event['operation'].startswith('inode_') or
-                                   event['operation'] in self.OP_TYPE_FILE_OR_NET):
+        if event['operation'] and (event['operation'].startswith('file_')
+                                   or event['operation'].startswith('inode_')
+                                   or event['operation'] in self.OP_TYPE_FILE_OR_NET):
             # file or network event?
             if event['family'] and event['protocol'] and event['sock_type']:
                 # 'unix' events also use keywords like 'connect', but protocol is 0 and should therefore be filtered out
                 return 'net'
-            elif event['denied_mask']:
+            elif event['denied_mask'] or event['operation'] == 'file_lock':
                 return 'file'
             else:
                 raise AppArmorException('unknown file or network event type')
