@@ -75,6 +75,7 @@ class SignalRule(BaseRule):
     ALL = __SignalAll
 
     rule_name = 'signal'
+    _match_re = RE_PROFILE_SIGNAL
 
     def __init__(self, access, signal, peer, audit=False, deny=False, allow_keyword=False,
                  comment='', log_event=None):
@@ -83,32 +84,24 @@ class SignalRule(BaseRule):
                          comment=comment, log_event=log_event)
 
         self.access, self.all_access, unknown_items = check_and_split_list(
-            access, access_keywords, SignalRule.ALL, 'SignalRule', 'access')
+            access, access_keywords, self.ALL, type(self).__name__, 'access')
         if unknown_items:
-            raise AppArmorException(_('Passed unknown access keyword to SignalRule: %s') % ' '.join(unknown_items))
+            raise AppArmorException(_('Passed unknown access keyword to %s: %s') % (type(self).__name__, ' '.join(unknown_items)))
 
         self.signal, self.all_signals, unknown_items = check_and_split_list(
-            signal, signal_keywords, SignalRule.ALL, 'SignalRule', 'signal')
+            signal, signal_keywords, self.ALL, type(self).__name__, 'signal')
         if unknown_items:
             for item in unknown_items:
                 if RE_SIGNAL_REALTIME.match(item):
                     self.signal.add(item)
                 else:
-                    raise AppArmorException(_('Passed unknown signal keyword to SignalRule: %s') % item)
+                    raise AppArmorException(_('Passed unknown signal keyword to %s: %s') % (type(self).__name__, item))
 
         self.peer, self.all_peers = self._aare_or_all(peer, 'peer', is_path=False, log_event=log_event)
 
     @classmethod
-    def _match(cls, raw_rule):
-        return RE_PROFILE_SIGNAL.search(raw_rule)
-
-    @classmethod
-    def _parse(cls, raw_rule):
-        """parse raw_rule and return SignalRule"""
-
-        matches = cls._match(raw_rule)
-        if not matches:
-            raise AppArmorException(_("Invalid signal rule '%s'") % raw_rule)
+    def _create_instance(cls, raw_rule, matches):
+        """parse raw_rule and return instance of this class"""
 
         audit, deny, allow_keyword, comment = parse_modifiers(matches)
 
@@ -127,7 +120,7 @@ class SignalRule(BaseRule):
                     access = access[1:-1]
                 access = access.replace(',', ' ').split()  # split by ',' or whitespace
             else:
-                access = SignalRule.ALL
+                access = cls.ALL
 
             if details.group('signal'):
                 signal = details.group('signal')
@@ -136,19 +129,19 @@ class SignalRule(BaseRule):
                 signal = RE_FILTER_QUOTES.sub(r' \1 ', signal)  # filter out quote pairs
                 signal = signal.replace(',', ' ').split()  # split at ',' or whitespace
             else:
-                signal = SignalRule.ALL
+                signal = cls.ALL
 
             if details.group('peer'):
                 peer = details.group('peer')
             else:
-                peer = SignalRule.ALL
+                peer = cls.ALL
         else:
-            access = SignalRule.ALL
-            signal = SignalRule.ALL
-            peer = SignalRule.ALL
+            access = cls.ALL
+            signal = cls.ALL
+            peer = cls.ALL
 
-        return SignalRule(access, signal, peer,
-                          audit=audit, deny=deny, allow_keyword=allow_keyword, comment=comment)
+        return cls(access, signal, peer,
+                   audit=audit, deny=deny, allow_keyword=allow_keyword, comment=comment)
 
     def get_clean(self, depth=0):
         """return rule (in clean/default formatting)"""
@@ -182,7 +175,7 @@ class SignalRule(BaseRule):
 
         return ('%s%ssignal%s%s%s,%s' % (space, self.modifiers_str(), access, signal, peer, self.comment))
 
-    def is_covered_localvars(self, other_rule):
+    def _is_covered_localvars(self, other_rule):
         """check if other_rule is covered by this rule object"""
 
         if not self._is_covered_list(self.access, self.all_access, other_rule.access, other_rule.all_access, 'access'):
@@ -197,12 +190,9 @@ class SignalRule(BaseRule):
         # still here? -> then it is covered
         return True
 
-    def is_equal_localvars(self, rule_obj, strict):
+    def _is_equal_localvars(self, rule_obj, strict):
         """compare if rule-specific variables are equal"""
 
-        if not type(rule_obj) == SignalRule:
-            raise AppArmorBug('Passed non-signal rule: %s' % str(rule_obj))
-
         if (self.access != rule_obj.access
                 or self.all_access != rule_obj.all_access):
             return False
@@ -216,16 +206,29 @@ class SignalRule(BaseRule):
 
         return True
 
-    def logprof_header_localvars(self):
+    def _logprof_header_localvars(self):
         access = logprof_value_or_all(self.access, self.all_access)
         signal = logprof_value_or_all(self.signal, self.all_signals)
         peer   = logprof_value_or_all(self.peer,   self.all_peers)  # noqa: E221
 
-        return [
+        return (
             _('Access mode'), access,
             _('Signal'), signal,
-            _('Peer'), peer
-        ]
+            _('Peer'), peer,
+        )
+
+    @staticmethod
+    def hashlog_from_event(hl, e):
+        hl[e['peer']][e['denied_mask']][e['signal']] = True
+
+    @classmethod
+    def from_hashlog(cls, hl):
+        for peer in hl.keys():
+            if '//null-' in peer:
+                continue  # ignore null-* peers
+
+            for access, signal in BaseRule.generate_rules_from_hashlog(hl[peer], 2):
+                yield cls(access, signal, peer, log_event=True)
 
 
 class SignalRuleset(BaseRuleset):
