@@ -21,17 +21,17 @@ from common_test import AATest, setup_all_loops
 
 class TestConvert_regexp(AATest):
     tests = (
-        ('/foo',          '^/foo$'),
-        ('/{foo,bar}',    '^/(foo|bar)$'),
-        # ('/\{foo,bar}',   '^/\{foo,bar}$'), # XXX gets converted to ^/\(foo|bar)$
-        ('/fo[abc]',      '^/fo[abc]$'),
-        ('/foo bar',      '^/foo bar$'),
-        ('/x\y',          '^/x\y$'),
-        ('/x\[y',         '^/x\[y$'),
-        ('/x\\y',         '^/x\\y$'),
-        ('/fo?',          '^/fo[^/\000]$'),
-        ('/foo/*',        '^/foo/(((?<=/)[^/\000]+)|((?<!/)[^/\000]*))$'),
-        ('/foo/**.bar',   '^/foo/(((?<=/)[^\000]+)|((?<!/)[^\000]*))\.bar$'),
+        ('/foo',        '^/foo$'),
+        ('/{foo,bar}',  '^/(foo|bar)$'),
+        # ('/\{foo,bar}', '^/\{foo,bar}$'),  # XXX gets converted to ^/\(foo|bar)$
+        ('/fo[abc]',    '^/fo[abc]$'),
+        ('/foo bar',    '^/foo bar$'),
+        (r'/x\y',       r'^/x\y$'),
+        (r'/x\[y',      r'^/x\[y$'),
+        ('/x\\y',       '^/x\\y$'),
+        ('/fo?',        '^/fo[^/\000]$'),
+        ('/foo/*',      '^/foo/(((?<=/)[^/\000]+)|((?<!/)[^/\000]*))$'),
+        ('/foo/**.bar', '^/foo/(((?<=/)[^\000]+)|((?<!/)[^\000]*))\\.bar$'),
     )
 
     def _run_test(self, params, expected):
@@ -138,12 +138,12 @@ class TestConvert_regexpAndAAREMatch(AATest):
     def _run_test(self, params, expected):
         regex, path = params
         parsed_regex = re.compile(convert_regexp(regex))
-        self.assertEqual(bool(parsed_regex.search(path)), expected, 'Incorrectly Parsed regex: %s' % regex)
+        self.assertEqual(bool(parsed_regex.search(path)), expected, 'Incorrectly Parsed regex: ' + regex)
 
         aare_obj = AARE(regex, True)
-        self.assertEqual(aare_obj.match(path), expected, 'Incorrectly parsed AARE object: %s' % regex)
+        self.assertEqual(aare_obj.match(path), expected, 'Incorrectly parsed AARE object: ' + regex)
         if not ('*' in path or '{' in path or '}' in path or '?' in path):
-            self.assertEqual(aare_obj.match(AARE(path, False)), expected, 'Incorrectly parsed AARE object: AARE(%s)' % regex)
+            self.assertEqual(aare_obj.match(AARE(path, False)), expected, 'Incorrectly parsed AARE object: AARE({})'.format(regex))
 
     def test_multi_usage(self):
         aare_obj = AARE('/foo/*', True)
@@ -211,12 +211,35 @@ class TestAAREIsEqual(AATest):
             aare_obj.is_equal(42)
 
 
+class TestAAREIs_eq(AATest):
+    tests = (
+        # regex     is path?  check for  expected
+        (('/foo',   True,     '/foo'),   True),
+        (('@{foo}', True,     '@{foo}'), True),
+        (('/**',    True,     '/foo'),   False),
+    )
+
+    def _run_test(self, params, expected):
+        regex, is_path, check_for = params
+        aare_obj_1 = AARE(regex, is_path)
+        aare_obj_2 = AARE(check_for, is_path)
+
+        self.assertEqual(aare_obj_1 == aare_obj_2, expected)
+        self.assertFalse(aare_obj_1 == check_for)
+
+    def test_is_eq_invalid_1(self):
+        aare_obj = AARE('/foo/**', True)
+        self.assertFalse(aare_obj == 42)
+
+
 class TestAAREIsPath(AATest):
     tests = (
         # regex       is path?  match for   expected
         (('/foo*',    True,     '/foobar'), True),
         (('@{PROC}/', True,     '/foobar'), False),
         (('foo*',     False,    'foobar'),  True),
+        (('{/a,/b}',  True,     '/a'),      True),
+        (('{@{X},/b}', True,    '/b'),      True),
     )
 
     def _run_test(self, params, expected):
@@ -228,6 +251,18 @@ class TestAAREIsPath(AATest):
         with self.assertRaises(AppArmorException):
             AARE('foo*', True)
 
+    def test_path_bad_alternative(self):
+        with self.assertRaises(AppArmorException):
+            AARE('{/a,@{foo},invalid}', True)
+
+    def test_path_bad_alternative2(self):
+        with self.assertRaises(AppArmorException):
+            AARE('{invalid,@{X},/X}', True)
+
+    def test_path_bad_alternative3(self):
+        with self.assertRaises(AppArmorException):
+            AARE('{/foo,@{invalid,/X}', True)
+
 
 class TestAARERepr(AATest):
     def test_repr(self):
