@@ -65,6 +65,8 @@ class TestLibapparmorTestMulti(AATest):
                         'src_name',  # pivotroot
                         'dbus_bus', 'dbus_interface', 'dbus_member', 'dbus_path',  # dbus
                         'peer_pid', 'peer_profile',  # dbus
+                        'net_addr', 'peer_addr',  # unix
+
                 ):
                     pass
                 elif parsed_items['operation'] == 'exec' and label in ('sock_type', 'family', 'protocol'):
@@ -151,8 +153,6 @@ log_to_skip = [
 
 # tests that do not produce the expected profile (checked with assertNotEqual)
 log_to_profile_known_failures = [
-    'testcase_pivotroot_01',  # pivot_rot not yet supported in logparser
-
     # exec events
     'testcase01',
     'testcase12',
@@ -173,7 +173,6 @@ log_to_profile_skip = [
 # tests that cause an empty log
 log_to_profile_known_empty_log = [
     'change_onexec_lp1648143',  # change_onexec not supported in logparser.py yet (and the log is about "no new privs" error)
-    'testcase_pivotroot_01',  # pivotroot not yet supported in logparser
     'ptrace_garbage_lp1689667_1',  # no denied= in log
     'ptrace_no_denied_mask',  # no denied= in log
     'unconfined-change_hat',  # unconfined trying to change_hat, which isn't allowed
