@@ -35,6 +35,9 @@ skip_startswith = (
 
     # Pux and Cux (which actually mean PUx and CUx) get rejected by the tools
     'generated_x/exact-',
+
+    # don't handle rule priorities yet
+    'file/priority/',
 )
 
 # testcases that should raise an exception, but don't
@@ -84,6 +87,8 @@ exception_not_raised = (
     'mount/bad_opt_31.sd',
     'mount/bad_1.sd',
     'mount/bad_2.sd',
+    'mount/bad_3.sd',
+    'mount/bad_4.sd',
 
     'profile/flags/flags_bad10.sd',
     'profile/flags/flags_bad11.sd',
@@ -153,6 +158,11 @@ exception_not_raised = (
     'profile/flags/flags_bad84.sd',
     'profile/flags/flags_bad85.sd',
     'profile/flags/flags_bad86.sd',
+    # flags=(error=EXX)
+    'profile/flags/flags_bad87.sd',
+    'profile/flags/flags_bad88.sd',
+    'profile/flags/flags_bad89.sd',
+
     'profile/flags/flags_bad_disconnected_path1.sd',
     'profile/flags/flags_bad_disconnected_path2.sd',
     'profile/flags/flags_bad_disconnected_path3.sd',
@@ -161,35 +171,16 @@ exception_not_raised = (
     'profile/profile_ns_bad8.sd',  # 'profile :ns/t' without terminating ':'
     'ptrace/bad_10.sd',  # peer with invalid regex
     'signal/bad_21.sd',  # invalid regex
-    'unix/bad_attr_1.sd',
-    'unix/bad_attr_2.sd',
-    'unix/bad_attr_3.sd',
-    'unix/bad_attr_4.sd',
-    'unix/bad_bind_1.sd',
-    'unix/bad_bind_2.sd',
-    'unix/bad_create_1.sd',
-    'unix/bad_create_2.sd',
-    'unix/bad_listen_1.sd',
-    'unix/bad_listen_2.sd',
-    'unix/bad_modifier_1.sd',
-    'unix/bad_modifier_2.sd',
-    'unix/bad_modifier_3.sd',
-    'unix/bad_modifier_4.sd',
-    'unix/bad_opt_1.sd',
-    'unix/bad_opt_2.sd',
-    'unix/bad_opt_3.sd',
-    'unix/bad_opt_4.sd',
-    'unix/bad_peer_1.sd',
+
+    # Invalid regexes
     'unix/bad_regex_01.sd',
     'unix/bad_regex_02.sd',
     'unix/bad_regex_03.sd',
     'unix/bad_regex_04.sd',
-    'unix/bad_shutdown_1.sd',
-    'unix/bad_shutdown_2.sd',
-    'unix/bad_peer_2.sd',
-    'unix/bad_attr_5.sd',
-    'unix/bad_opt_5.sd',
-    'unix/bad_shutdown_3.sd',
+
+    'unix/bad_modifier_2.sd',  # We do not check for duplicated keywords
+    'unix/bad_bind_2.sd',  # We do not check bind coherency
+
     'vars/vars_bad_3.sd',
     'vars/vars_bad_4.sd',
     'vars/vars_bad_5.sd',
@@ -233,6 +224,8 @@ exception_not_raised = (
     'xtrans/simple_bad_conflicting_x_6.sd',
     'xtrans/simple_bad_conflicting_x_8.sd',
     'xtrans/x-conflict.sd',
+
+    'network/perms/bad_modifier_2.sd',
 )
 
 # testcases with lines that don't match any regex and end up as "unknown line"
@@ -314,14 +307,12 @@ unknown_line = (
     'bare_include_tests/ok_85.sd',
     'bare_include_tests/ok_86.sd',
 
-    # According to spec mount should be in the form fstype=... options=... and NOT in the form options=... fstype=...
-    'mount/ok_opt_combo_3.sd',
-    'mount/ok_opt_combo_2.sd',
-    'mount/ok_opt_combo_1.sd',
-    'mount/ok_opt_combo_4.sd',
+    # Unsupported \\" in unix AARE
+    'unix/ok_regex_03.sd',
+    'unix/ok_regex_09.sd',
+    'unix/ok_regex_13.sd',
+    'unix/ok_regex_19.sd',
 
-    # Options should be comma separated
-    'mount/in_4.sd',  # also order option then fstype is invalid
 )
 
 # testcases with various unexpected failures
@@ -419,104 +410,23 @@ syntax_failure = (
     'generated_perms_leading/dominate-Cuxtarget.sd',
     'generated_perms_leading/dominate-ownerPuxtarget2.sd',
 
-    # escaping with \
+    # escaping with "\"
     'file/ok_embedded_spaces_4.sd',  # \-escaped space
     'file/file/ok_embedded_spaces_4.sd',  # \-escaped space
     'file/ok_quoted_4.sd',  # quoted string including \"
 
+    # mount rules with multiple 'options' or 'fstype' are not supported by the tools yet, and when writing them, only the last 'options'/'fstype' would survive.
+    # Therefore MountRule intentionally raises an exception when parsing such a rule.
+    'mount/ok_opt_87.sd',  # multiple options
+    'mount/ok_opt_88.sd',  # multiple fstype
+
     # misc
-    'vars/vars_dbus_8.sd',  # Path doesn't start with / or variable: {/@{TLDS}/foo,/com/@{DOMAINS}}
+    'vars/vars_dbus_12.sd',  # AARE starting with {{ are not handled
     'vars/vars_simple_assignment_12.sd',  # Redefining existing variable @{BAR} ('\' not handled)
     'bare_include_tests/ok_2.sd',  # two #include<...> in one line
 
-    # fine grained net
-    'network/network_ok_8.sd',
-    'network/network_ok_9.sd',
-    'network/network_ok_10.sd',
-    'network/network_ok_11.sd',
-    'network/network_ok_12.sd',
-    'network/network_ok_13.sd',
-    'network/network_ok_14.sd',
-    'network/network_ok_15.sd',
-    'network/network_ok_16.sd',
+    # network port range
     'network/network_ok_17.sd',
-    'network/network_ok_18.sd',
-    'network/network_ok_19.sd',
-    'network/network_ok_20.sd',
-    'network/network_ok_21.sd',
-    'network/network_ok_22.sd',
-    'network/network_ok_23.sd',
-    'network/network_ok_24.sd',
-    'network/network_ok_25.sd',
-    'network/network_ok_26.sd',
-    'network/network_ok_27.sd',
-    'network/network_ok_28.sd',
-    'network/network_ok_29.sd',
-    'network/network_ok_30.sd',
-    'network/network_ok_31.sd',
-    'network/network_ok_32.sd',
-    'network/network_ok_33.sd',
-    'network/network_ok_34.sd',
-    'network/network_ok_35.sd',
-    'network/network_ok_36.sd',
-    'network/network_ok_37.sd',
-    'network/network_ok_38.sd',
-    'network/network_ok_39.sd',
-    'network/network_ok_40.sd',
-    'network/network_ok_41.sd',
-    'network/network_ok_42.sd',
-    'network/network_ok_43.sd',
-    'network/network_ok_44.sd',
-    'network/perms/ok_accept_1.sd',
-    'network/perms/ok_accept_2.sd',
-    'network/perms/ok_attr_1.sd',
-    'network/perms/ok_attr_2.sd',
-    'network/perms/ok_attr_3.sd',
-    'network/perms/ok_attr_4.sd',
-    'network/perms/ok_attr_5.sd',
-    'network/perms/ok_attr_6.sd',
-    'network/perms/ok_attr_7.sd',
-    'network/perms/ok_attr_8.sd',
-    'network/perms/ok_bind_1.sd',
-    'network/perms/ok_bind_2.sd',
-    'network/perms/ok_connect_1.sd',
-    'network/perms/ok_connect_2.sd',
-    'network/perms/ok_create_1.sd',
-    'network/perms/ok_create_2.sd',
-    'network/perms/ok_create_3.sd',
-    'network/perms/ok_create_4.sd',
-    'network/perms/ok_listen_1.sd',
-    'network/perms/ok_listen_2.sd',
-    'network/perms/ok_listen_3.sd',
-    'network/perms/ok_msg_1.sd',
-    'network/perms/ok_msg_2.sd',
-    'network/perms/ok_msg_3.sd',
-    'network/perms/ok_msg_4.sd',
-    'network/perms/ok_msg_5.sd',
-    'network/perms/ok_msg_6.sd',
-    'network/perms/ok_msg_7.sd',
-    'network/perms/ok_msg_8.sd',
-    'network/perms/ok_msg_9.sd',
-    'network/perms/ok_msg_10.sd',
-    'network/perms/ok_msg_12.sd',
-    'network/perms/ok_msg_13.sd',
-    'network/perms/ok_msg_14.sd',
-    'network/perms/ok_msg_15.sd',
-    'network/perms/ok_msg_16.sd',
-    'network/perms/ok_msg_17.sd',
-    'network/perms/ok_msg_18.sd',
-    'network/perms/ok_msg_19.sd',
-    'network/perms/ok_msg_20.sd',
-    'network/perms/ok_opt_1.sd',
-    'network/perms/ok_opt_2.sd',
-    'network/perms/ok_opt_3.sd',
-    'network/perms/ok_opt_4.sd',
-    'network/perms/ok_opt_5.sd',
-    'network/perms/ok_opt_6.sd',
-    'network/perms/ok_opt_7.sd',
-    'network/perms/ok_shutdown_1.sd',
-    'network/perms/ok_shutdown_2.sd',
-    'network/perms/ok_shutdown_3.sd',
 )
 
 
