@@ -17,12 +17,13 @@ import apparmor.aa
 from apparmor.common import AppArmorBug, AppArmorException
 from apparmor.profile_list import ProfileList
 from apparmor.profile_storage import ProfileStorage
+from apparmor.aare import AARE
 from apparmor.rule.abi import AbiRule
 from apparmor.rule.alias import AliasRule
 from apparmor.rule.boolean import BooleanRule
 from apparmor.rule.include import IncludeRule
 from apparmor.rule.variable import VariableRule
-from common_test import AATest, setup_aa, setup_all_loops, write_file
+from common_test import AATest, setup_aa, setup_all_loops, write_file, skip_active_profiles
 
 
 class TestAdd_profile(AATest):
@@ -33,28 +34,28 @@ class TestAdd_profile(AATest):
     def testEmpty(self):
         self.assertEqual(self.pl.profile_names, {})
         self.assertEqual(self.pl.attachments, {})
-        self.assertEqual('%s' % self.pl, "\n".join(['', '<ProfileList>', '', '</ProfileList>', '']))
+        self.assertEqual(str(self.pl), "\n".join(['', '<ProfileList>', '', '</ProfileList>', '']))
 
     def testAdd_profile_1(self):
         self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)
         self.assertEqual(self.pl.profile_names, {'foo': '/etc/apparmor.d/bin.foo'})
-        self.assertEqual(self.pl.attachments, {'/bin/foo': {'f': '/etc/apparmor.d/bin.foo', 'p': 'foo'}})
+        self.assertEqual(self.pl.attachments, {'/bin/foo': {'f': '/etc/apparmor.d/bin.foo', 'p': 'foo', 're': AARE('/bin/foo', True)}})
         self.assertEqual(self.pl.profiles_in_file('/etc/apparmor.d/bin.foo'), ['foo'])
-        self.assertEqual('%s' % self.pl, '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')
+        self.assertEqual(str(self.pl), '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')
 
     def testAdd_profile_2(self):
         self.pl.add_profile('/etc/apparmor.d/bin.foo', None, '/bin/foo', self.dummy_profile)
         self.assertEqual(self.pl.profile_names, {})
-        self.assertEqual(self.pl.attachments, {'/bin/foo': {'f': '/etc/apparmor.d/bin.foo', 'p': '/bin/foo'}})
+        self.assertEqual(self.pl.attachments, {'/bin/foo': {'f': '/etc/apparmor.d/bin.foo', 'p': '/bin/foo', 're': AARE('/bin/foo', True)}})
         self.assertEqual(self.pl.profiles_in_file('/etc/apparmor.d/bin.foo'), ['/bin/foo'])
-        self.assertEqual('%s' % self.pl, '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')
+        self.assertEqual(str(self.pl), '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')
 
     def testAdd_profile_3(self):
         self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', None, self.dummy_profile)
         self.assertEqual(self.pl.profile_names, {'foo': '/etc/apparmor.d/bin.foo'})
         self.assertEqual(self.pl.attachments, {})
         self.assertEqual(self.pl.profiles_in_file('/etc/apparmor.d/bin.foo'), ['foo'])
-        self.assertEqual('%s' % self.pl, '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')
+        self.assertEqual(str(self.pl), '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')
 
     def testAdd_profileError_1(self):
         with self.assertRaises(AppArmorBug):
@@ -153,6 +154,7 @@ class TestFilename_from_attachment(AATest):
         with self.assertRaises(AppArmorBug):
             self.pl.filename_from_attachment('foo')
 
+
 class TestProfile_from_attachment(TestFilename_from_attachment):
     # uses AASetup from TestFilename_from_attachment
     tests = (
@@ -197,9 +199,9 @@ class TestAdd_inc_ie(AATest):
         self.assertEqual(list(self.pl.files.keys()), [])
 
     def test_dedup_inc_ie_1(self):
-        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.parse('include <tunables/global>'))
-        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.parse('#include if exists <tunables/global>  # comment'))
-        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.parse('   #include         <tunables/global>    '))
+        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.create_instance('include <tunables/global>'))
+        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.create_instance('#include if exists <tunables/global>  # comment'))
+        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.create_instance('   #include         <tunables/global>    '))
         deleted = self.pl.delete_preamble_duplicates('/etc/apparmor.d/bin.foo')
         self.assertEqual(deleted, 2)
         self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
@@ -235,8 +237,8 @@ class TestAdd_abi(AATest):
         self.assertEqual(list(self.pl.files.keys()), [])
 
     def test_dedup_abi_1(self):
-        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule.parse('abi <abi/4.19>,'))
-        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule.parse('   abi     <abi/4.19>  ,  # comment'))
+        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule.create_instance('abi <abi/4.19>,'))
+        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule.create_instance('   abi     <abi/4.19>  ,  # comment'))
         self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
         deleted = self.pl.delete_preamble_duplicates('/etc/apparmor.d/bin.foo')
         self.assertEqual(deleted, 1)
@@ -317,9 +319,9 @@ class TestAdd_variable(AATest):
         self.assertEqual(list(self.pl.files.keys()), [])
 
     def test_dedup_variable_1(self):
-        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.parse('@{foo} = /foo'))
-        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.parse('@{foo} += /bar  # comment'))
-        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.parse('@{foo}    += /bar /baz'))
+        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.create_instance('@{foo} = /foo'))
+        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.create_instance('@{foo} += /bar  # comment'))
+        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.create_instance('@{foo}    += /bar /baz'))
         deleted = self.pl.delete_preamble_duplicates('/etc/apparmor.d/bin.foo')
         self.assertEqual(deleted, 1)
         self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
@@ -375,7 +377,7 @@ class AaTest_get_all_merged_variables(AATest):
         self.createTmpdir()
 
         # copy the local profiles to the test directory
-        self.profile_dir = '%s/profiles' % self.tmpdir
+        self.profile_dir = self.tmpdir + '/profiles'
         apparmor.aa.profile_dir = self.profile_dir
         shutil.copytree('../../profiles/apparmor.d/', self.profile_dir, symlinks=True)
 
@@ -385,7 +387,9 @@ class AaTest_get_all_merged_variables(AATest):
         # load the profiles and abstractions
         apparmor.aa.profile_dir = self.profile_dir
         apparmor.aa.loadincludes()
-        apparmor.aa.read_profiles()
+        # Keep this synced with the list in test-logprof.py and
+        # test-minitools.py
+        apparmor.aa.read_profiles(skip_profiles=skip_active_profiles)
 
     def test_unchanged(self):
         self._load_profiles()
