@@ -48,7 +48,7 @@ def cmd(command, input=None, stderr=subprocess.STDOUT, stdout=subprocess.PIPE, s
 (rc, output, outerr) = cmd(('../../common/list_capabilities.sh',))
 if rc != 0:
     sys.stderr.write("make list_capabilities failed: " + output + outerr)
-    exit(rc)
+    sys.exit(rc)
 
 capabilities = re.sub('CAP_', '', output.strip()).lower().split('\n')
 benign_caps = []
@@ -60,7 +60,7 @@ for cap in capabilities:
 (rc, output, outerr) = cmd(('../../common/list_af_names.sh',))
 if rc != 0:
     sys.stderr.write("make list_af_names failed: " + output + outerr)
-    exit(rc)
+    sys.exit(rc)
 
 af_names = []
 af_pairs = re.sub('AF_', '', output.strip()).lower().split(",")
@@ -77,6 +77,7 @@ for af_pair in af_pairs:
 aa_network_types = r'\s+tcp|\s+udp|\s+icmp'
 
 aa_flags = ('complain',
+            'unconfined',
             'audit',
             'attach_disconnected',
             'no_attach_disconnected',
@@ -173,7 +174,7 @@ sys.stdout.write('" do not edit this file - edit apparmor.vim.in or create-appar
 with open("apparmor.vim.in") as template:
     for line in template:
         line = re.sub(regex, my_repl, line.rstrip())
-        sys.stdout.write('%s\n' % line)
+        sys.stdout.write(line + '\n')
 
 sys.stdout.write("\n\n\n\n")
 
